@@ -1,4 +1,4 @@
-<?
+<?php
 /**
  * Common DokuWiki functions
  *
@@ -6,100 +6,12 @@
  * @author     Andreas Gohr <andi@splitbrain.org>
  */
 
-  require_once("conf/dokuwiki.php");
-  require_once("inc/io.php");
-  require_once('inc/utf8.php');
-  require_once('inc/mail.php');
-
-  //set up error reporting to sane values
-  error_reporting(E_ALL ^ E_NOTICE);
-
-  //make session rewrites XHTML compliant
-  ini_set('arg_separator.output', '&amp;');
-
-  //init session
-  session_name("DokuWiki");
-  session_start();
-
-  //kill magic quotes
-  if (get_magic_quotes_gpc()) {
-    if (!empty($_GET))    remove_magic_quotes($_GET);
-    if (!empty($_POST))   remove_magic_quotes($_POST);
-    if (!empty($_COOKIE)) remove_magic_quotes($_COOKIE);
-    if (!empty($_REQUEST)) remove_magic_quotes($_REQUEST);
-    if (!empty($_SESSION)) remove_magic_quotes($_SESSION);
-    ini_set('magic_quotes_gpc', 0);
-  }
-  set_magic_quotes_runtime(0);
-  ini_set('magic_quotes_sybase',0);
-
-  //disable gzip if not available
-  if($conf['usegzip'] && !function_exists('gzopen')){
-    $conf['usegzip'] = 0;
-  }
-
-  //remember original umask
-  $conf['oldumask'] = umask();
-
-  //make absolute mediaweb
-  if(!preg_match('#^(https?://|/)#i',$conf['mediaweb'])){
-    $conf['mediaweb'] = getBaseURL().$conf['mediaweb'];
-  }
-
-/**
- * remove magic quotes recursivly
- *
- * @author Andreas Gohr <andi@splitbrain.org>
- */
-function remove_magic_quotes(&$array) {
-  foreach (array_keys($array) as $key) {
-    if (is_array($array[$key])) {
-      remove_magic_quotes($array[$key]);
-    }else {
-      $array[$key] = stripslashes($array[$key]);
-    }
-  } 
-} 
-
-/**
- * Returns the full absolute URL to the directory where
- * DokuWiki is installed in (includes a trailing slash)
- *
- * @author Andreas Gohr <andi@splitbrain.org>
- */
-function getBaseURL($abs=false){
-  global $conf;
-  //if canonical url enabled always return absolute
-  if($conf['canonical']) $abs = true;
-
-  $dir = dirname($_SERVER['PHP_SELF']).'/';
-
-  $dir = str_replace('\\','/',$dir); #bugfix for weird WIN behaviour
-  $dir = preg_replace('#//+#','/',$dir);
-
-  //finish here for relative URLs
-  if(!$abs) return $dir;
-
-  $port = ':'.$_SERVER['SERVER_PORT'];
-  //remove port from hostheader as sent by IE
-  $host = preg_replace('/:.*$/','',$_SERVER['HTTP_HOST']);
-
-  // see if HTTPS is enabled - apache leaves this empty when not available,
-  // IIS sets it to 'off', 'false' and 'disabled' are just guessing
-  if (preg_match('/^(|off|false|disabled)$/i',$_SERVER['HTTPS'])){
-    $proto = 'http://';
-    if ($_SERVER['SERVER_PORT'] == '80') {
-      $port='';
-    }
-  }else{
-    $proto = 'https://';
-    if ($_SERVER['SERVER_PORT'] == '443') {
-      $port='';
-    }
-  }
-
-  return $proto.$host.$port.$dir;
-}
+  if(!defined('DOKU_INC')) define('DOKU_INC',realpath(dirname(__FILE__).'/../').'/');
+  require_once(DOKU_CONF.'dokuwiki.php');
+  require_once(DOKU_INC.'inc/io.php');
+  require_once(DOKU_INC.'inc/utf8.php');
+  require_once(DOKU_INC.'inc/mail.php');
+  require_once(DOKU_INC.'inc/parserutils.php');
 
 /**
  * Return info about the current document as associative
@@ -114,12 +26,12 @@ function pageinfo(){
   global $conf;
 
   if($_SERVER['REMOTE_USER']){
-    $info['user']     = $_SERVER['REMOTE_USER'];
-    $info['userinfo'] = $USERINFO;
-    $info['perm']     = auth_quickaclcheck($ID);
+    $info['userinfo']   = $USERINFO;
+    $info['perm']       = auth_quickaclcheck($ID);
+    $info['subscribed'] = is_subscribed($ID,$_SERVER['REMOTE_USER']);
   }else{
-    $info['user']     = '';
-    $info['perm']     = auth_aclcheck($ID,'',null);
+    $info['perm']       = auth_aclcheck($ID,'',null);
+    $info['subscribed'] = false;
   }
 
   $info['namespace'] = getNS($ID);
@@ -135,6 +47,7 @@ function pageinfo(){
       $REV = '';
     }
   }
+  $info['rev'] = $REV;
   if($info['exists']){
     $info['writable'] = (is_writable($info['filepath']) &&
                          ($info['perm'] >= AUTH_EDIT));
@@ -153,12 +66,50 @@ function pageinfo(){
   $info['ip']     = $revinfo['ip'];
   $info['user']   = $revinfo['user'];
   $info['sum']    = $revinfo['sum'];
-  $info['editor'] = $revinfo['ip'];
-  if($revinfo['user']) $info['editor'].= ' ('.$revinfo['user'].')';
+
+  if($revinfo['user']){
+    $info['editor'] = $revinfo['user'];
+  }else{
+    $info['editor'] = $revinfo['ip'];
+  }
 
   return $info;
 }
 
+/**
+ * Build an string of URL parameters
+ *
+ * @author Andreas Gohr
+ */
+function buildURLparams($params){
+  $url = '';
+  $amp = false;
+  foreach($params as $key => $val){
+    if($amp) $url .= '&amp;';
+
+    $url .= $key.'=';
+    $url .= urlencode($val);
+    $amp = true;
+  }
+  return $url;
+}
+
+/**
+ * Build an string of html tag attributes
+ *
+ * @author Andreas Gohr
+ */
+function buildAttributes($params){
+  $url = '';
+  foreach($params as $key => $val){
+    $url .= $key.'="';
+    $url .= htmlspecialchars ($val);
+    $url .= '" ';
+  }
+  return $url;
+}
+
+
 /**
  * print a message
  *
@@ -188,7 +139,11 @@ function msg($message,$lvl=0){
   }else{
     $MSG = array();
     $MSG[]=array('lvl' => $errors[$lvl], 'msg' => $message);
-    html_msgarea();
+    if(function_exists('html_msgarea')){
+      html_msgarea();
+    }else{
+      print "ERROR($lvl) $message";
+    }
   }
 }
 
@@ -198,6 +153,10 @@ function msg($message,$lvl=0){
  * @author Andreas Gohr <andi@splitbrain.org>
  */
 function breadcrumbs(){
+  // we prepare the breadcrumbs early for quick session closing
+  static $crumbs = null;
+  if($crumbs != null) return $crumbs;
+
   global $ID;
   global $ACT;
   global $conf;
@@ -208,18 +167,29 @@ function breadcrumbs(){
     $crumbs = array();
   }
   //we only save on show and existing wiki documents
-  if($ACT != 'show' || !@file_exists(wikiFN($ID))){
+  $file = wikiFN($ID);
+  if($ACT != 'show' || !@file_exists($file)){
     $_SESSION[$conf['title']]['bc'] = $crumbs;
     return $crumbs;
   }
+
+  // page names
+  $name = noNS($ID);
+  if ($conf['useheading']) {
+    // get page title
+    $title = p_get_first_heading($ID);
+    if ($title) {
+      $name = $title;
+    }
+  }
+
   //remove ID from array
-  $pos = array_search($ID,$crumbs);
-  if($pos !== false && $pos !== null){
-    array_splice($crumbs,$pos,1);
+  if (isset($crumbs[$ID])) {
+    unset($crumbs[$ID]);
   }
 
   //add to array
-  $crumbs[] =$ID;
+  $crumbs[$ID] = $name;
   //reduce size
   while(count($crumbs) > $conf['breadcrumbs']){
     array_shift($crumbs);
@@ -257,59 +227,115 @@ function idfilter($id,$ue=true){
 }
 
 /**
- * This builds a link to a wikipage (using getBaseURL)
+ * This builds a link to a wikipage
+ *
+ * It handles URL rewriting and adds additional parameter if
+ * given in $more
  *
  * @author Andreas Gohr <andi@splitbrain.org>
  */
-function wl($id='',$more='',$script='doku.php',$canonical=false){
+function wl($id='',$more='',$abs=false){
   global $conf;
-  $more = str_replace(',','&amp;',$more);
+  if(is_array($more)){
+    $more = buildURLparams($more);
+  }else{
+    $more = str_replace(',','&amp;',$more);
+  }
 
   $id    = idfilter($id);
-  $xlink = getBaseURL($canonical);
-
-  if(!$conf['userewrite']){
-    $xlink .= $script;
-    $xlink .= '?id='.$id;
-    if($more) $xlink .= '&amp;'.$more;
+  if($abs){
+    $xlink = DOKU_URL;
   }else{
+    $xlink = DOKU_BASE;
+  }
+
+  if($conf['userewrite'] == 2){
+    $xlink .= DOKU_SCRIPT.'/'.$id;
+    if($more) $xlink .= '?'.$more;
+  }elseif($conf['userewrite']){
     $xlink .= $id;
     if($more) $xlink .= '?'.$more;
+  }else{
+    $xlink .= DOKU_SCRIPT.'?id='.$id;
+    if($more) $xlink .= '&amp;'.$more;
   }
   
   return $xlink;
 }
 
+/**
+ * Build a link to a media file
+ *
+ * Will return a link to the detail page if $direct is false
+ */
+function ml($id='',$more='',$direct=true){
+  global $conf;
+  if(is_array($more)){
+    $more = buildURLparams($more);
+  }else{
+    $more = str_replace(',','&amp;',$more);
+  }
+
+  $xlink = DOKU_BASE;
+
+  // external URLs are always direct without rewriting
+  if(preg_match('#^(https?|ftp)://#i',$id)){
+    $xlink .= 'lib/exe/fetch.php';
+    if($more){
+      $xlink .= '?'.$more;
+      $xlink .= '&amp;media='.urlencode($id);
+    }else{
+      $xlink .= '?media='.urlencode($id);
+    }
+    return $xlink;
+  } 
+
+  $id = idfilter($id);
+
+  // decide on scriptname
+  if($direct){
+    if($conf['userewrite'] == 1){
+      $script = '_media';
+    }else{
+      $script = 'lib/exe/fetch.php';
+    }
+  }else{
+    if($conf['userewrite'] == 1){
+      $script = '_detail';
+    }else{
+      $script = 'lib/exe/detail.php';
+    }
+  }
+
+  // build URL based on rewrite mode
+   if($conf['userewrite']){
+     $xlink .= $script.'/'.$id;
+     if($more) $xlink .= '?'.$more;
+   }else{
+     if($more){
+       $xlink .= $script.'?'.$more;
+       $xlink .= '&amp;media='.$id;
+     }else{
+       $xlink .= $script.'?media='.$id;
+     }
+   }
+
+  return $xlink;
+}
+
+
+
 /**
  * Just builds a link to a script
  *
+ * @todo   maybe obsolete
  * @author Andreas Gohr <andi@splitbrain.org>
  */
 function script($script='doku.php'){
-  $link = getBaseURL();
-  $link .= $script;
-  return $link;
-}
-
-/**
- * Return namespacepart of a wiki ID
- *
- * @author Andreas Gohr <andi@splitbrain.org>
- */
-function getNS($id){
- if(strpos($id,':')!==false){
-   return substr($id,0,strrpos($id,':'));
- }
- return false;
-}
-
-/**
- * Returns the ID without the namespace
- *
- * @author Andreas Gohr <andi@splitbrain.org>
- */
-function noNS($id){
-  return preg_replace('/.*:/','',$id);
+#  $link = getBaseURL();
+#  $link .= $script;
+#  return $link;
+  return DOKU_BASE.DOKU_SCRIPT;
 }
 
 /**
@@ -326,7 +352,7 @@ function checkwordblock(){
 
   if(!$conf['usewordblock']) return false;
 
-  $blockfile = file('conf/wordblock.conf');
+  $blockfile = file(DOKU_CONF.'wordblock.conf');
   //how many lines to read at once (to work around some PCRE limits)
   if(version_compare(phpversion(),'4.3.0','<')){
     //old versions of PCRE define a maximum of parenthesises even if no
@@ -428,84 +454,6 @@ function unlock($id){
   return false;
 }
 
-/**
- * Remove unwanted chars from ID
- *
- * Cleans a given ID to only use allowed characters. Accented characters are
- * converted to unaccented ones
- *
- * @author Andreas Gohr <andi@splitbrain.org>
- */
-function cleanID($id){
-  global $conf;
-  global $lang;
-  $id = trim($id);
-  $id = utf8_strtolower($id);
-
-  //alternative namespace seperator
-  $id = strtr($id,';',':');
-  if($conf['useslash']){
-    $id = strtr($id,'/',':');
-  }else{
-    $id = strtr($id,'/','_');
-  }
-
-  if($conf['deaccent']) $id = utf8_deaccent($id,-1);
-
-  //remove specials
-  //$id = preg_replace('#[\x00-\x20 ¡!"§$%&()\[\]{}¿\\?`\'\#~*+=,<>\|^°@µ¹²³¼½¬]#u','_',$id);
-  $id = utf8_stripspecials($id,'_','_:.-');
-
-  //clean up
-  $id = preg_replace('#__#','_',$id);
-  $id = preg_replace('#:+#',':',$id);
-  $id = trim($id,':._-');
-  $id = preg_replace('#:[:\._\-]+#',':',$id);
-
-  return($id);
-}
-
-/**
- * returns the full path to the datafile specified by ID and
- * optional revision
- *
- * The filename is URL encoded to protect Unicode chars
- *
- * @author Andreas Gohr <andi@splitbrain.org>
- */
-function wikiFN($id,$rev=''){
-  global $conf;
-  $id = cleanID($id);
-  $id = str_replace(':','/',$id);
-  if(empty($rev)){
-    $fn = $conf['datadir'].'/'.$id.'.txt';
-  }else{
-    $fn = $conf['olddir'].'/'.$id.'.'.$rev.'.txt';
-    if($conf['usegzip'] && !@file_exists($fn)){
-      //return gzip if enabled and plaintext doesn't exist
-      $fn .= '.gz';
-    }
-  }
-  $fn = utf8_encodeFN($fn);
-  return $fn;
-}
-
-/**
- * Returns the full filepath to a localized textfile if local
- * version isn't found the english one is returned
- *
- * @author Andreas Gohr <andi@splitbrain.org>
- */
-function localeFN($id){
-  global $conf;
-  $file = './lang/'.$conf['lang'].'/'.$id.'.txt';
-  if(!@file_exists($file)){
-    //fall back to english
-    $file = './lang/en/'.$id.'.txt';
-  }
-  return cleanText($file);
-}
-
 /**
  * convert line ending to unix format
  *
@@ -530,23 +478,6 @@ function formText($text){
   return htmlspecialchars($text);
 }
 
-/**
- * Returns the specified local text in parsed format
- *
- * @author Andreas Gohr <andi@splitbrain.org>
- */
-function parsedLocale($id){
-  //disable section editing
-  global $parser;
-  $se = $parser['secedit'];
-  $parser['secedit'] = false;
-  //fetch parsed locale
-  $html = io_cacheParse(localeFN($id));
-  //reset section editing
-  $parser['secedit'] = $se;
-  return $html;
-}
-
 /**
  * Returns the specified local text in raw format
  *
@@ -556,39 +487,6 @@ function rawLocale($id){
   return io_readFile(localeFN($id));
 }
 
-
-/**
- * Returns the parsed Wikitext for the given id and revision.
- *
- * If $excuse is true an explanation is returned if the file
- * wasn't found
- *
- * @author Andreas Gohr <andi@splitbrain.org>
- */
-function parsedWiki($id,$rev='',$excuse=true){
-  $file = wikiFN($id,$rev);
-  $ret  = '';
-  
-  //ensure $id is in global $ID (needed for parsing)
-  global $ID;
-  $ID = $id;
-  
-  if($rev){
-    if(@file_exists($file)){
-      $ret = parse(io_readFile($file));
-    }elseif($excuse){
-      $ret = parsedLocale('norev');
-    }
-  }else{
-    if(@file_exists($file)){
-      $ret = io_cacheParse($file);
-    }elseif($excuse){
-      $ret = parsedLocale('newpage');
-    }
-  }
-  return $ret;
-}
-
 /**
  * Returns the raw WikiText
  *
@@ -598,11 +496,32 @@ function rawWiki($id,$rev=''){
   return io_readFile(wikiFN($id,$rev));
 }
 
+/**
+ * Returns the pagetemplate contents for the ID's namespace
+ *
+ * @author Andreas Gohr <andi@splitbrain.org>
+ */
+function pageTemplate($id){
+  global $conf;
+  global $INFO;
+  $tpl = io_readFile(dirname(wikiFN($id)).'/_template.txt');
+  $tpl = str_replace('@ID@',$id,$tpl);
+  $tpl = str_replace('@NS@',getNS($id),$tpl);
+  $tpl = str_replace('@PAGE@',strtr(noNS($id),'_',' '),$tpl);
+  $tpl = str_replace('@USER@',$_SERVER['REMOTE_USER'],$tpl);
+  $tpl = str_replace('@NAME@',$INFO['userinfo']['name'],$tpl);
+  $tpl = str_replace('@MAIL@',$INFO['userinfo']['mail'],$tpl);
+  $tpl = str_replace('@DATE@',date($conf['dformat']),$tpl);
+  return $tpl;
+}
+
+
 /**
  * Returns the raw Wiki Text in three slices.
  *
  * The range parameter needs to have the form "from-to"
- * and gives the range of the section.
+ * and gives the range of the section in bytes - no
+ * UTF-8 awareness is needed.
  * The returned order is prefix, section and suffix.
  *
  * @author Andreas Gohr <andi@splitbrain.org>
@@ -610,13 +529,12 @@ function rawWiki($id,$rev=''){
 function rawWikiSlices($range,$id,$rev=''){
   list($from,$to) = split('-',$range,2);
   $text = io_readFile(wikiFN($id,$rev));
-  $text = split("\n",$text);
   if(!$from) $from = 0;
-  if(!$to)   $to   = count($text);
+  if(!$to)   $to   = strlen($text)+1;
 
-  $slices[0] = join("\n",array_slice($text,0,$from));
-  $slices[1] = join("\n",array_slice($text,$from,$to + 1  - $from));
-  $slices[2] = join("\n",array_slice($text,$to+1));
+  $slices[0] = substr($text,0,$from-1);
+  $slices[1] = substr($text,$from-1,$to-$from);
+  $slices[2] = substr($text,$to);
 
   return $slices;
 }
@@ -662,7 +580,6 @@ function dbg($msg,$hidden=false){
  */
 function addLogEntry($date,$id,$summary=""){
   global $conf;
-  $id     = cleanID($id);//FIXME not needed anymore?
 
   if(!@is_writable($conf['changelog'])){
     msg($conf['changelog'].' is not writable!',-1);
@@ -674,53 +591,124 @@ function addLogEntry($date,$id,$summary=""){
   $user   = $_SERVER['REMOTE_USER'];
 
   $logline = join("\t",array($date,$remote,$id,$user,$summary))."\n";
+  io_saveFile($conf['changelog'],$logline,true);
+}
 
-  //FIXME: use adjusted io_saveFile instead
-  $fh = fopen($conf['changelog'],'a');
-  if($fh){
-    fwrite($fh,$logline);
-    fclose($fh);
+/**
+ * Internal function used by getRecents
+ *
+ * don't call directly
+ *
+ * @see getRecents()
+ * @author Andreas Gohr <andi@splitbrain.org>
+ */
+function _handleRecent($line,$incdel,$ns,$subNS){
+  static $seen  = array();         //caches seen pages and skip them
+  if(empty($line)) return false;   //skip empty lines
+
+  // split the line into parts
+  list($dt,$ip,$id,$usr,$sum) = explode("\t",$line);
+  
+  // skip seen ones
+  if($seen[$id]) return false;
+
+  // remember in seen to skip additional sights
+  $seen[$id] = 1;
+
+  // filter namespace
+  if (($ns) && (strpos($id,$ns.':') !== 0)) return false;
+
+  // exclude subnamespaces
+  if ((!$subNS) && (getNS($id) != $ns)) return false;
+
+  // check existance
+  if(!@file_exists(wikiFN($id))){
+    if(!$incdel){
+      return false;
+    }else{
+      $recent = array();
+      $recent['del'] = true;
+    }
+  }else{
+    $recent = array();
+    $recent['del'] = false;
   }
+
+  $recent['id']   = $id;
+  $recent['date'] = $dt;
+  $recent['ip']   = $ip;
+  $recent['user'] = $usr;
+  $recent['sum']  = $sum;
+
+  return $recent;
 }
 
 /**
  * returns an array of recently changed files using the
  * changelog
  *
+ * @param int    $first   number of first entry returned (for paginating
+ * @param int    $num     return $num entries
+ * @param bool   $incdel  include deleted pages?
+ * @param string $ns      restrict to given namespace
+ * @param bool   $subNS   include subnamespaces
+ *
  * @author Andreas Gohr <andi@splitbrain.org>
  */
-function getRecents($num=0,$incdel=false){
+function getRecents($first,$num,$incdel=false,$ns='',$subNS=true){
   global $conf;
   $recent = array();
-  if(!$num) $num = $conf['recent'];
+  $count  = 0;
+
+  if(!$num)
+    return $recent;
 
   if(!@is_readable($conf['changelog'])){
     msg($conf['changelog'].' is not readable',-1);
     return $recent;
   }
 
-  $loglines = file($conf['changelog']);
-  rsort($loglines); //reverse sort on timestamp
+  $fh  = fopen($conf['changelog'],'r');
+  $buf = '';
+  $csz = 4096;                              //chunksize
+  fseek($fh,0,SEEK_END);                    // jump to the end
+  $pos = ftell($fh);                        // position pointer
 
-  foreach ($loglines as $line){
-    $line = rtrim($line);        //remove newline
-    if(empty($line)) continue;   //skip empty lines
-    $info = split("\t",$line);   //split into parts
-    //add id if not in yet and file still exists and is allowed to read
-    if(!$recent[$info[2]] && 
-       (@file_exists(wikiFN($info[2])) || $incdel) &&
-       (auth_quickaclcheck($info[2]) >= AUTH_READ)
-      ){
-      $recent[$info[2]]['date'] = $info[0];
-      $recent[$info[2]]['ip']   = $info[1];
-      $recent[$info[2]]['user'] = $info[3];
-      $recent[$info[2]]['sum']  = $info[4];
-      $recent[$info[2]]['del']  = !@file_exists(wikiFN($info[2]));
+  // now read backwards into buffer
+  while($pos > 0){
+    $pos -= $csz;                           // seek to previous chunk...
+    if($pos < 0) $pos = 0;                  // ...or rest of file
+    fseek($fh,$pos);
+
+    $buf = fread($fh,$csz).$buf;            // prepend to buffer
+
+    $lines = explode("\n",$buf);            // split buffer into lines
+
+    if($pos > 0){
+      $buf = array_shift($lines);           // first one may be still incomplete
     }
-    if(count($recent) >= $num){
-      break; //finish if enough items found
+
+    $cnt = count($lines);
+    if(!$cnt) continue;                     // no lines yet
+
+    // handle lines
+    for($i = $cnt-1; $i >= 0; $i--){
+      $rec = _handleRecent($lines[$i],$incdel,$ns,$subNS);
+      if($rec !== false){
+        if(--$first >= 0) continue;         // skip first entries
+        $recent[] = $rec;
+        $count++;
+
+        // break while when we have enough entries
+        if($count >= $num){
+          $pos = 0; // will break the while loop
+          break;    // will break the for loop
+        }
+      }
     }
-  }
+  }// end of while
+
+  fclose($fh);
   return $recent;
 }
 
@@ -732,6 +720,9 @@ function getRecents($num=0,$incdel=false){
  */
 function getRevisionInfo($id,$rev){
   global $conf;
+
+  if(!$rev) return(null);
+
   $info = array();
   if(!@is_readable($conf['changelog'])){
     msg($conf['changelog'].' is not readable',-1);
@@ -739,7 +730,7 @@ function getRevisionInfo($id,$rev){
   }
   $loglines = file($conf['changelog']);
   $loglines = preg_grep("/$rev\t\d+\.\d+\.\d+\.\d+\t$id\t/",$loglines);
-  rsort($loglines); //reverse sort on timestamp (shouldn't be needed)
+  $loglines = array_reverse($loglines); //reverse sort on timestamp (shouldn't be needed)
   $line = split("\t",$loglines[0]);
   $info['date'] = $line[0];
   $info['ip']   = $line[1];
@@ -766,11 +757,18 @@ function saveWikiText($id,$text,$summary){
   $old  = saveOldRevision($id);
 
   if (empty($text)){
-    // remove empty files
+    // remove empty file
     @unlink($file);
+    // remove any meta info
+    $mfiles = metaFiles($id);
+    foreach ($mfiles as $mfile) {
+      if (file_exists($mfile)) @unlink($mfile);
+    }
     $del = true;
     //autoset summary on deletion
     if(empty($summary)) $summary = $lang['deleted'];
+    //remove empty namespaces
+    io_sweepNS($id);
   }else{
     // save file (datadir is created in io_saveFile)
     io_saveFile($file,$text);
@@ -778,11 +776,13 @@ function saveWikiText($id,$text,$summary){
   }
 
   addLogEntry(@filemtime($file),$id,$summary);
-  notify($id,$old,$summary);
+  // send notify mails
+  notify($id,'admin',$old,$summary);
+  notify($id,'subscribers',$old,$summary);
   
   //purge cache on add by updating the purgefile
   if($conf['purgeonadd'] && (!$old || $del)){
-    io_saveFile($conf['datadir'].'/.cache/purgefile',time());
+    io_saveFile($conf['cachedir'].'/purgefile',time());
   }
 }
 
@@ -809,30 +809,49 @@ function saveOldRevision($id){
 }
 
 /**
- * Sends a notify mail to the wikiadmin when a page was
- * changed
+ * Sends a notify mail on page change
+ *
+ * @param  string $id       The changed page
+ * @param  string $who      Who to notify (admin|subscribers)
+ * @param  int    $rev      Old page revision
+ * @param  string $summary  What changed
  *
  * @author Andreas Gohr <andi@splitbrain.org>
  */
-function notify($id,$rev="",$summary=""){
+function notify($id,$who,$rev='',$summary=''){
   global $lang;
   global $conf;
-  $hdrs ='';
-  if(empty($conf['notify'])) return; //notify enabled?
+
+  // decide if there is something to do
+  if($who == 'admin'){
+    if(empty($conf['notify'])) return; //notify enabled?
+    $text = rawLocale('mailtext');
+    $to   = $conf['notify'];
+    $bcc  = '';
+  }elseif($who == 'subscribers'){
+    if(!$conf['subscribers']) return; //subscribers enabled?
+    $bcc  = subscriber_addresslist($id);
+    if(empty($bcc)) return;
+    $to   = '';
+    $text = rawLocale('subscribermail');
+  }else{
+    return; //just to be safe
+  }
   
-  $text = rawLocale('mailtext');
   $text = str_replace('@DATE@',date($conf['dformat']),$text);
   $text = str_replace('@BROWSER@',$_SERVER['HTTP_USER_AGENT'],$text);
   $text = str_replace('@IPADDRESS@',$_SERVER['REMOTE_ADDR'],$text);
   $text = str_replace('@HOSTNAME@',gethostbyaddr($_SERVER['REMOTE_ADDR']),$text);
-  $text = str_replace('@NEWPAGE@',wl($id,'','doku.php',true),$text);
-  $text = str_replace('@DOKUWIKIURL@',getBaseURL(true),$text);
+  $text = str_replace('@NEWPAGE@',wl($id,'',true),$text);
+  $text = str_replace('@PAGE@',$id,$text);
+  $text = str_replace('@TITLE@',$conf['title'],$text);
+  $text = str_replace('@DOKUWIKIURL@',DOKU_URL,$text);
   $text = str_replace('@SUMMARY@',$summary,$text);
   $text = str_replace('@USER@',$_SERVER['REMOTE_USER'],$text);
   
   if($rev){
     $subject = $lang['mail_changed'].' '.$id;
-    $text = str_replace('@OLDPAGE@',wl($id,"rev=$rev",'doku.php',true),$text);
+    $text = str_replace('@OLDPAGE@',wl($id,"rev=$rev",true),$text);
     require_once("inc/DifferenceEngine.php");
     $df  = new Diff(split("\n",rawWiki($id,$rev)),
                     split("\n",rawWiki($id)));
@@ -844,8 +863,9 @@ function notify($id,$rev="",$summary=""){
     $diff = rawWiki($id);
   }
   $text = str_replace('@DIFF@',$diff,$text);
+  $subject = '['.$conf['title'].'] '.$subject;
 
-  mail_send($conf['notify'],$subject,$text,$conf['mailfrom']);
+  mail_send($to,$subject,$text,$conf['mailfrom'],'',$bcc);
 }
 
 /**
@@ -858,6 +878,7 @@ function getRevisions($id){
   $revs = array();
   $clid = cleanID($id);
   if(strrpos($clid,':')) $clid = substr($clid,strrpos($clid,':')+1); //remove path
+  $clid = utf8_encodeFN($clid);
 
   if (is_dir($revd) && $dh = opendir($revd)) {
     while (($file = readdir($dh)) !== false) {
@@ -872,34 +893,15 @@ function getRevisions($id){
   return $revs;
 }
 
-/**
- * downloads a file from the net and saves it to the given location
- *
- * @author Andreas Gohr <andi@splitbrain.org>
- */
-function download($url,$file){
-  $fp = @fopen($url,"rb");
-  if(!$fp) return false;
-
-  while(!feof($fp)){
-    $cont.= fread($fp,1024);
-  }
-  fclose($fp);
-
-  $fp2 = @fopen($file,"w");
-  if(!$fp2) return false;
-  fwrite($fp2,$cont);
-  fclose($fp2);
-  return true;
-} 
-
 /**
  * extracts the query from a google referer
  *
+ * @todo   should be more generic and support yahoo et al
  * @author Andreas Gohr <andi@splitbrain.org>
  */
 function getGoogleQuery(){
   $url = parse_url($_SERVER['HTTP_REFERER']);
+  if(!$url) return '';
 
   if(!preg_match("#google\.#i",$url['host'])) return '';
   $query = array();
@@ -952,7 +954,32 @@ function filesize_h($size, $dec = 1){
 }
 
 /**
- * Run a few sanity checks
+ * return an obfuscated email address in line with $conf['mailguard'] setting
+ *
+ * @author Harry Fuecks <hfuecks@gmail.com>
+ * @author Christopher Smith <chris@jalakai.co.uk>
+ */
+function obfuscate($email) {
+  global $conf;
+  
+  switch ($conf['mailguard']) {
+    case 'visible' :
+      $obfuscate = array('@' => ' [at] ', '.' => ' [dot] ', '-' => ' [dash] ');
+      return strtr($email, $obfuscate);
+      
+    case 'hex' :
+      $encode = '';
+      for ($x=0; $x < strlen($email); $x++) $encode .= '&#x' . bin2hex($email{$x}).';';
+      return $encode;
+      
+    case 'none' :
+    default :
+      return $email;
+  }            
+}
+
+/**
+ * Return DokuWikis version
  *
  * @author Andreas Gohr <andi@splitbrain.org>
  */
@@ -960,7 +987,7 @@ function getVersion(){
   //import version string
   if(@file_exists('VERSION')){
     //official release
-    return 'Release '.io_readfile('VERSION');
+    return 'Release '.trim(io_readfile(DOKU_INC.'/VERSION'));
   }elseif(is_dir('_darcs')){
     //darcs checkout
     $inv = file('_darcs/inventory');
@@ -1016,10 +1043,16 @@ function check(){
     msg('Mediadir is not writable',-1);
   }
 
-  if(is_writable('conf/users.auth')){
-    msg('conf/users.auth is writable',1);
+  if(is_writable($conf['cachedir'])){
+    msg('Cachedir is writable',1);
   }else{
-    msg('conf/users.auth is not writable',0);
+    msg('Cachedir is not writable',-1);
+  }
+
+  if(is_writable(DOKU_CONF.'users.auth.php')){
+    msg('conf/users.auth.php is writable',1);
+  }else{
+    msg('conf/users.auth.php is not writable',0);
   }
 
   if(function_exists('mb_strpos')){
@@ -1046,4 +1079,68 @@ function check(){
     msg('The current page is not writable you',0);
   }
 }
-?>
+
+/**
+ * Let us know if a user is tracking a page
+ *
+ * @author Andreas Gohr <andi@splitbrain.org>
+ */
+function is_subscribed($id,$uid){
+  $file=metaFN($id,'.mlist');
+  if (@file_exists($file)) {
+    $mlist = file($file);
+    $pos = array_search($uid."\n",$mlist);
+    return is_int($pos);
+  }
+
+  return false;
+}
+
+/**
+ * Return a string with the email addresses of all the
+ * users subscribed to a page
+ *
+ * @author Steven Danz <steven-danz@kc.rr.com>
+ */
+function subscriber_addresslist($id){
+  global $conf;
+
+  $emails = '';
+  
+  if (!$conf['subscribers']) return;
+
+  $mlist = array();
+  $file=metaFN($id,'.mlist');
+  if (file_exists($file)) {
+    $mlist = file($file);
+  }
+  if(count($mlist) > 0) {
+    foreach ($mlist as $who) {
+      $who = rtrim($who);
+      $info = auth_getUserData($who);
+      $level = auth_aclcheck($id,$who,$info['grps']);
+      if ($level >= AUTH_READ) {
+        if (strcasecmp($info['mail'],$conf['notify']) != 0) {
+          if (empty($emails)) {
+            $emails = $info['mail'];
+          } else {
+            $emails = "$emails,".$info['mail'];
+          }
+        }
+      }
+    }
+  }
+
+  return $emails;
+}
+
+/**
+ * Removes quoting backslashes
+ *
+ * @author Andreas Gohr <andi@splitbrain.org>
+ */
+function unslash($string,$char="'"){
+  return str_replace('\\'.$char,$char,$string);
+}
+
+//Setup VIM: ex: et ts=2 enc=utf-8 :
