@@ -1,4 +1,4 @@
-<?
+<?php
 /**
  * Mail functions
  *
@@ -6,9 +6,11 @@
  * @author     Andreas Gohr <andi@splitbrain.org>
  */
 
-require_once('inc/utf8.php');
+  if(!defined('DOKU_INC')) define('DOKU_INC',realpath(dirname(__FILE__).'/../').'/');
+  require_once(DOKU_INC.'inc/utf8.php');
 
-define('MAILHEADER_EOL',"\n"); //end of line for mail headers
+  define('MAILHEADER_EOL',"\n"); //end of line for mail headers
+  #define('MAILHEADER_ASCIIONLY',1);
 
 /**
  * UTF-8 autoencoding replacement for PHPs mail function
@@ -17,8 +19,6 @@ define('MAILHEADER_EOL',"\n"); //end of line for mail headers
  * like this: 'Andreas Gohr <andi@splitbrain.org>' - the text part is encoded
  * automatically. You can seperate receivers by commas.
  *
- * @todo  currently an empty To: header is added
- *
  * @param string $to      Receiver of the mail (multiple seperated by commas)
  * @param string $subject Mailsubject
  * @param string $body    Messagebody
@@ -32,36 +32,48 @@ define('MAILHEADER_EOL',"\n"); //end of line for mail headers
  * @see    mail()
  */
 function mail_send($to, $subject, $body, $from='', $cc='', $bcc='', $headers=null, $params=null){
-  if(!utf8_isASCII($subject)) $subject = '=?UTF-8?Q?'.mail_quotedprintable_encode($subject).'?=';
+  if(defined('MAILHEADER_ASCIIONLY')){
+    $subject = utf8_deaccent($subject);
+    $subject = utf8_strip($subject);
+  }
+
+  if(!utf8_isASCII($subject))
+    $subject = '=?UTF-8?Q?'.mail_quotedprintable_encode($subject).'?=';
 
   $header  = '';
+
+  $to = mail_encode_address($to);
   $header .= mail_encode_address($from,'From');
-  $header .= mail_encode_address($to,'To');
   $header .= mail_encode_address($cc,'Cc');
   $header .= mail_encode_address($bcc,'Bcc');
   $header .= 'MIME-Version: 1.0'.MAILHEADER_EOL;
   $header .= 'Content-Type: text/plain; charset=UTF-8'.MAILHEADER_EOL;
   $header .= 'Content-Transfer-Encoding: quoted-printable'.MAILHEADER_EOL;
   $header .= $headers;
-  $heade   = trim($header);
+  $header  = trim($header);
 
   $body = mail_quotedprintable_encode($body);
 
-  return @mail(null,$subject,$body,$header,$params);
+  if($params == null){
+    return @mail($to,$subject,$body,$header);
+  }else{
+    return @mail($to,$subject,$body,$header,$params);
+  }
 }
 
 /**
  * Encodes an email address header
  *
- * Unicode chracters will be encoded quoted_printable for headers like
+ * Unicode characters will be deaccented and encoded
+ * quoted_printable for headers.
  * Addresses may not contain Non-ASCII data!
  *
  * Example:
- *   mail_encode_address("föö <foo@bar.com>, me@somewhere.com","To");
+ *   mail_encode_address("föö <foo@bar.com>, me@somewhere.com","TBcc");
  *
  * @param string $string Multiple headers seperated by commas
  */
-function mail_encode_address($string,$header='To'){
+function mail_encode_address($string,$header=''){
   $headers = '';
   $parts = split(',',$string);
   foreach ($parts as $part){
@@ -91,21 +103,31 @@ function mail_encode_address($string,$header='To'){
       continue;
     }
 
-    // no text was given
-    if(empty($text)){
-      $headers .= $header.': <'.$addr.'>'.MAILHEADER_EOL;
-      continue;
-    }
+    // text was given
+    if(!empty($text)){
+      // add address quotes
+      $addr = "<$addr>";
 
-    // FIME: can make problems with long headers?
-    if(!utf8_isASCII($text)){
-      $text = '=?UTF-8?Q?'.mail_quotedprintable_encode($text).'?=';
+      if(defined('MAILHEADER_ASCIIONLY')){
+        $text = utf8_deaccent($text);
+        $text = utf8_strip($text);
+      }
+
+      if(!utf8_isASCII($text)){
+        $text = '=?UTF-8?Q?'.mail_quotedprintable_encode($text).'?=';
+      }
     }
     
-    //construct header
-    $headers .= $header.': '.$text.' <'.$addr.'>'.MAILHEADER_EOL;
+    // add to header comma seperated and in new line to avoid too long headers
+    if($headers != '') $headers .= ','.MAILHEADER_EOL.' ';
+    $headers .= $text.$addr;
   }
 
+  if(empty($headers)) return null;
+
+  //if headername was given add it and close correctly
+  if($header) $headers = $header.': '.$headers.MAILHEADER_EOL;
+
   return $headers;
 }
 
@@ -120,7 +142,7 @@ function mail_encode_address($string,$header='To'){
  * @return  bool          true if address is valid
  */
 function mail_isvalid($email){
-  return eregi("^[0-9a-z]([-_.]?[0-9a-z])*@[0-9a-z]([-.]?[0-9a-z])*\\.[a-z]{2,4}$", $email);
+  return eregi("^[0-9a-z]([+-_.]?[0-9a-z])*@[0-9a-z]([-.]?[0-9a-z])*\\.[a-z]{2,4}$", $email);
 }
 
 /**
@@ -173,4 +195,5 @@ function mail_quotedprintable_encode($input='',$line_max=74,$space_conv=false){
 }
 
 
-?>
+
+//Setup VIM: ex: et ts=2 enc=utf-8 :
