@@ -1,4 +1,4 @@
-<?
+<?php
 /**
  * DokuWiki search functions
  *
@@ -6,7 +6,8 @@
  * @author     Andreas Gohr <andi@splitbrain.org>
  */
 
-  require_once("inc/common.php");
+  if(!defined('DOKU_INC')) define('DOKU_INC',realpath(dirname(__FILE__).'/../').'/');
+  require_once(DOKU_INC.'inc/common.php');
 
 /**
  * recurse direcory
@@ -24,10 +25,13 @@ function search(&$data,$base,$func,$opts,$dir='',$lvl=1){
   $dh = @opendir($base.'/'.$dir);
   if(!$dh) return;
   while(($file = readdir($dh)) !== false){
-    if(preg_match('/^\./',$file)) continue; //skip hidden files and upper dirs
+    if(preg_match('/^[\._]/',$file)) continue; //skip hidden files and upper dirs
     if(is_dir($base.'/'.$dir.'/'.$file)){
       $dirs[] = $dir.'/'.$file;
       continue;
+    }elseif(substr($file,-5) == '.lock'){
+      //skip lockfiles
+      continue;
     }
     $files[] = $dir.'/'.$file;
   }
@@ -69,6 +73,38 @@ function search(&$data,$base,$func,$opts,$dir='',$lvl=1){
  * one deeper nested)
  */
 
+/**
+ * Searches for pages beginning with the given query
+ *
+ * @author Andreas Gohr <andi@splitbrain.org>
+ */
+function search_qsearch(&$data,$base,$file,$type,$lvl,$opts){
+  $item = array();
+
+  if($type == 'd'){
+    return false; //no handling yet
+  }
+
+  //get id
+  $id = pathID($file);
+
+  //check if it matches the query
+  if(!preg_match('/^'.preg_quote($opts['query'],'/').'/u',$id)){
+    return false;
+  }
+
+  //check ACL
+  if(auth_quickaclcheck($id) < AUTH_READ){
+    return false;
+  }
+
+  $data[]=array( 'id'    => $id,
+                 'type'  => $type,
+                 'level' => 1,
+                 'open'  => true);
+  return true;
+}
+
 /**
  * Build the browsable index of pages
  *
@@ -127,7 +163,7 @@ function search_media(&$data,$base,$file,$type,$lvl,$opts){
   if($type == 'd') return false;
 
   $info         = array();
-  $info['id']   = pathID($file);
+  $info['id']   = pathID($file,true);
 
   //check ACL for namespace (we have no ACL for mediafiles)
   if(auth_quickaclcheck(getNS($info['id']).':*') < AUTH_READ){
@@ -138,7 +174,8 @@ function search_media(&$data,$base,$file,$type,$lvl,$opts){
   $info['size'] = filesize($base.'/'.$file);
   if(preg_match("/\.(jpe?g|gif|png)$/",$file)){
     $info['isimg'] = true;
-    $info['info']  = getimagesize($base.'/'.$file);
+    require_once(DOKU_INC.'inc/JpegMeta.php');
+    $info['meta']  = new JpegMeta($base.'/'.$file);
   }else{
     $info['isimg'] = false;
   }
@@ -180,15 +217,31 @@ function search_pagename(&$data,$base,$file,$type,$lvl,$opts){
   if(!preg_match('#\.txt$#',$file)) return true;
 
   //simple stringmatching 
-  if(strpos($file,$opts['query']) !== false){
-    //check ACL
-    $id = pathID($file);
-    if(auth_quickaclcheck($id) < AUTH_READ){
-      return false;
-    } 
-    $data[]['id'] = $id;
+  if (!empty($opts['query'])){
+    if(strpos($file,$opts['query']) !== false){
+      //check ACL
+      $id = pathID($file);
+      if(auth_quickaclcheck($id) < AUTH_READ){
+        return false;
+      } 
+      $data[]['id'] = $id;
+    }
   }
+  return true;
+}
 
+/**
+ * Just lists all documents
+ *
+ * @author  Andreas Gohr <andi@splitbrain.org>
+ */
+function search_allpages(&$data,$base,$file,$type,$lvl,$opts){
+  //we do nothing with directories
+  if($type == 'd') return true;
+  //only search txt files
+  if(!preg_match('#\.txt$#',$file)) return true;
+
+  $data[]['id'] = pathID($file);
   return true;
 }
 
@@ -199,6 +252,7 @@ function search_pagename(&$data,$base,$file,$type,$lvl,$opts){
  * $opts['name']  name of the page without namespace
  *
  * @author  Andreas Gohr <andi@splitbrain.org>
+ * @deprecated Replaced by ft_backlinks()
  */
 function search_backlinks(&$data,$base,$file,$type,$lvl,$opts){
   //we do nothing with directories
@@ -206,13 +260,10 @@ function search_backlinks(&$data,$base,$file,$type,$lvl,$opts){
   //only search txt files
   if(!preg_match('#\.txt$#',$file)) return true;;
 
-  //get text
-  $text = io_readfile($base.'/'.$file);
-
   //absolute search id
   $sid = cleanID($opts['ns'].':'.$opts['name']);
 
-  //construct current namespace
+  //current id and namespace
   $cid = pathID($file);
   $cns = getNS($cid);
 
@@ -221,31 +272,25 @@ function search_backlinks(&$data,$base,$file,$type,$lvl,$opts){
     return false;
   }
 
-  //match all links
-  //FIXME may be incorrect because of code blocks
-  //      CamelCase isn't supported, too
-  preg_match_all('#\[\[(.+?)\]\]#si',$text,$matches,PREG_SET_ORDER);
-  foreach($matches as $match){
-    //get ID from link and discard most non wikilinks
-    list($mid) = split('\|',$match[1],2);
-    if(preg_match("#^(https?|telnet|gopher|file|wais|ftp|ed2k|irc)://#",$mid)) continue;
-    if(preg_match("#\w+>#",$mid)) continue;
-    $mns = getNS($mid);
-   	//namespace starting with "." - prepend current namespace
-    if(strpos($mns,'.')===0){
-      $mid = $cns.":".substr($mid,1);
-    }
-    if($mns===false){
-      //no namespace in link? add current
-      $mid = "$cns:$mid";
-    }
-    $mid = cleanID($mid);
+  //fetch instructions
+  require_once(DOKU_INC.'inc/parserutils.php');
+  $instructions = p_cached_instructions($base.$file,true);
+  if(is_null($instructions)) return false;
 
-    if ($mid == $sid){
-      $data[]['id'] = $cid;
-      break;
+  //check all links for match
+  foreach($instructions as $ins){
+    if($ins[0] == 'internallink' || ($conf['camelcase'] && $ins[0] == 'camelcaselink') ){
+      $mid = $ins[1][0];
+      resolve_pageid($cns,$mid,$exists); //exists is not used 
+      if($mid == $sid){
+        //we have a match - finish
+        $data[]['id'] = $cid;
+        break;
+      }
     }
   }
+
+  return false;
 }
 
 /**
@@ -254,6 +299,7 @@ function search_backlinks(&$data,$base,$file,$type,$lvl,$opts){
  * $opts['query'] is the search query
  *
  * @author  Andreas Gohr <andi@splitbrain.org>
+ * @deprecated - fulltext indexer is used instead
  */
 function search_fulltext(&$data,$base,$file,$type,$lvl,$opts){
   //we do nothing with directories
@@ -267,22 +313,108 @@ function search_fulltext(&$data,$base,$file,$type,$lvl,$opts){
     return false;
   }
 
+  //create regexp from queries  
+  $poswords = array();
+  $negwords = array();
+  $qpreg = preg_split('/\s+/',$opts['query']);
+  
+  foreach($qpreg as $word){
+    switch(substr($word,0,1)){
+      case '-':
+        if(strlen($word) > 1){  // catch single '-'
+          array_push($negwords,preg_quote(substr($word,1),'#'));
+        }
+        break;
+      case '+':
+        if(strlen($word) > 1){  // catch single '+'
+          array_push($poswords,preg_quote(substr($word,1),'#'));
+        }
+        break;
+      default:
+        array_push($poswords,preg_quote($word,'#'));
+        break;
+    }
+  }
+
+  // a search without any posword is useless
+  if (!count($poswords)) return true;
+  
+  $reg  = '^(?=.*?'.join(')(?=.*?',$poswords).')';
+  $reg .= count($negwords) ? '((?!'.join('|',$negwords).').)*$' : '.*$';
+  search_regex($data,$base,$file,$reg,$poswords);
+  return true;
+}
+
+/**
+ * Reference search
+ * This fuction searches for existing references to a given media file
+ * and returns an array with the found pages. It doesn't pay any
+ * attention to ACL permissions to find every reference. The caller
+ * must check if the user has the appropriate rights to see the found
+ * page and eventually have to prevent the result from displaying.
+ *
+ * @param array  $data Reference to the result data structure
+ * @param string $base Base usually $conf['datadir']
+ * @param string $file current file or directory relative to $base
+ * @param char   $type Type either 'd' for directory or 'f' for file
+ * @param int    $lvl  Current recursion depht
+ * @param mixed  $opts option array as given to search()
+ *
+ * $opts['query'] is the demanded media file name
+ *
+ * @author  Andreas Gohr <andi@splitbrain.org>
+ * @author  Matthias Grimm <matthiasgrimm@users.sourceforge.net>
+ */
+function search_reference(&$data,$base,$file,$type,$lvl,$opts){
+  global $conf;
+  
+  //we do nothing with directories
+  if($type == 'd') return true;
+  
+  //only search txt files
+  if(!preg_match('#\.txt$#',$file)) return true;
+
+  //we finish after 'cnt' references found. The return value
+  //'false' will skip subdirectories to speed search up.
+  $cnt = $conf['refshow'] > 0 ? $conf['refshow'] : 1;
+  if(count($data) >= $cnt) return false;
+  
+  $reg = '\{\{ *\:?'.$opts['query'].' *(\|.*)?\}\}';
+  search_regex($data,$base,$file,$reg,array($opts['query']));
+  return true;
+}
+
+/* ------------- helper functions below -------------- */
+
+/**
+ * fulltext search helper
+ * searches a text file with a given regular expression
+ * no ACL checks are performed. This have to be done by
+ * the caller if necessary.
+ *
+ * @param array  $data  reference to array for results
+ * @param string $base  base directory
+ * @param string $file  file name to search in
+ * @param string $reg   regular expression to search for
+ * @param array  $words words that should be marked in the results
+ *
+ * @author  Andreas Gohr <andi@splitbrain.org>
+ * @author  Matthias Grimm <matthiasgrimm@users.sourceforge.net>
+ *
+ * @deprecated - fulltext indexer is used instead
+ */
+function search_regex(&$data,$base,$file,$reg,$words){
+
   //get text
   $text = io_readfile($base.'/'.$file);
   //lowercase text (u modifier does not help with case)
   $lctext = utf8_strtolower($text);
 
-  //create regexp from queries  
-  $qpreg = preg_split('/\s+/',preg_quote($opts['query'],'#'));
-  $qpreg = '('.join('|',$qpreg).')';
-
   //do the fulltext search
   $matches = array();
-  if($cnt = preg_match_all('#'.$qpreg.'#usi',$lctext,$matches)){
+  if($cnt = preg_match_all('#'.$reg.'#usi',$lctext,$matches)){
     //this is not the best way for snippet generation but the fastest I could find
-    //split query and only use the first token
-    $q = preg_split('/\s+/',$opts['query'],2);
-    $q = $q[0];
+    $q = $words[0];  //use first word for snippet creation
     $p = utf8_strpos($lctext,$q);
     $f = $p - 100;
     $l = utf8_strlen($q) + 200;
@@ -290,18 +422,21 @@ function search_fulltext(&$data,$base,$file,$type,$lvl,$opts){
     $snippet = '<span class="search_sep"> ... </span>'.
                htmlspecialchars(utf8_substr($text,$f,$l)).
                '<span class="search_sep"> ... </span>';
-    $snippet = preg_replace('#'.$qpreg.'#si','<span class="search_hit">\\1</span>',$snippet);
+    $mark    = '('.join('|', $words).')';
+    $snippet = preg_replace('#'.$mark.'#si','<span class="search_hit">\\1</span>',$snippet);
 
     $data[] = array(
-      'id'      => $id,
-      'count'   => $cnt,
-      'snippet' => $snippet,
+      'id'       => pathID($file),
+      'count'    => preg_match_all('#'.$mark.'#usi',$lctext,$matches),
+      'poswords' => join(' ',$words),
+      'snippet'  => $snippet,
     );
   }
-
+ 
   return true;
 }
 
+
 /**
  * fulltext sort
  *
@@ -324,14 +459,16 @@ function sort_search_fulltext($a,$b){
  * translates a document path to an ID
  *
  * @author  Andreas Gohr <andi@splitbrain.org>
+ * @todo    move to pageutils
  */
-function pathID($path){
+function pathID($path,$keeptxt=false){
   $id = utf8_decodeFN($path);
   $id = str_replace('/',':',$id);
-  $id = preg_replace('#\.txt$#','',$id);
+  if(!$keeptxt) $id = preg_replace('#\.txt$#','',$id);
   $id = preg_replace('#^:+#','',$id);
   $id = preg_replace('#:+$#','',$id);
   return $id;
 }
 
-?>
+
+//Setup VIM: ex: et ts=2 enc=utf-8 :
