@@ -1,8 +1,8 @@
-<?
+<?php
 /**
  * UTF8 helper functions
  *
- * @license    GPL 2 (http://www.gnu.org/licenses/gpl.html)
+ * @license    LGPL (http://www.gnu.org/copyleft/lesser.html)
  * @author     Andreas Gohr <andi@splitbrain.org>
  */
 
@@ -53,6 +53,23 @@ function utf8_isASCII($str){
   return true;
 }
 
+/**
+ * Strips all highbyte chars
+ *
+ * Returns a pure ASCII7 string
+ *
+ * @author Andreas Gohr <andi@splitbrain.org>
+ */
+function utf8_strip($str){
+  $ascii = '';
+  for($i=0; $i<strlen($str); $i++){
+    if(ord($str{$i}) <128){
+      $ascii .= $str{$i};
+    }
+  }
+  return $ascii;
+}
+
 /**
  * Tries to detect if a string is in Unicode encoding
  *
@@ -77,37 +94,131 @@ function utf8_check($Str) {
 }
 
 /**
- * This is a unicode aware replacement for strlen()
+ * Unicode aware replacement for strlen()
  *
- * Uses mb_string extension if available
+ * utf8_decode() converts characters that are not in ISO-8859-1
+ * to '?', which, for the purpose of counting, is alright - It's
+ * even faster than mb_strlen.
  *
- * @author Andreas Gohr <andi@splitbrain.org>
+ * @author <chernyshevsky at hotmail dot com>
  * @see    strlen()
+ * @see    utf8_decode()
  */
 function utf8_strlen($string){
-  if(!defined('UTF8_NOMBSTRING') && function_exists('mb_strlen'))
-    return mb_strlen($string,'utf-8');
-
-  $uni = utf8_to_unicode($string);
-  return count($uni);
+  return strlen(utf8_decode($string));
 }
 
 /**
- * This is a unicode aware replacement for substr()
+ * Unicode aware replacement for substr()
  *
- * Uses mb_string extension if available
- *
- * @author Andreas Gohr <andi@splitbrain.org>
+ * @author lmak at NOSPAM dot iti dot gr
+ * @link   http://www.php.net/manual/en/function.substr.php
  * @see    substr()
  */
-function utf8_substr($str, $start, $length=null){
-  if(!defined('UTF8_NOMBSTRING') && function_exists('mb_substr'))
-    return mb_substr($str,$start,$length,'utf-8');
+function utf8_substr($str,$start,$length=null){
+   preg_match_all("/./u", $str, $ar);
 
-  $uni = utf8_to_unicode($str);
-  return unicode_to_utf8(array_slice($uni,$start,$length));
+   if($length != null) {
+       return join("",array_slice($ar[0],$start,$length));
+   } else {
+       return join("",array_slice($ar[0],$start));
+   }
+} 
+
+/**
+ * Unicode aware replacement for substr_replace()
+ *
+ * @author Andreas Gohr <andi@splitbrain.org>
+ * @see    substr_replace()
+ */
+function utf8_substr_replace($string, $replacement, $start , $length=0 ){
+  $ret = '';
+  if($start>0) $ret .= utf8_substr($string, 0, $start);
+  $ret .= $replacement;
+  $ret .= utf8_substr($string, $start+$length);
+  return $ret;
 }
 
+/**
+ * Unicode aware replacement for explode
+ *
+ * @TODO   support third limit arg 
+ * @author Harry Fuecks <hfuecks@gmail.com>
+ * @see    explode();
+ */
+function utf8_explode($sep, $str) {
+  if ( $sep == '' ) {
+    trigger_error('Empty delimiter',E_USER_WARNING);
+    return FALSE;
+  }
+
+  return preg_split('!'.preg_quote($sep,'!').'!u',$str);
+}
+
+/**
+ * Unicode aware replacement for strrepalce()
+ *
+ * @todo   support PHP5 count (fourth arg)
+ * @author Harry Fuecks <hfuecks@gmail.com>
+ * @see    strreplace();
+ */
+function utf8_str_replace($s,$r,$str){
+  if(!is_array($s)){
+    $s = '!'.preg_quote($s,'!').'!u';
+  }else{
+    foreach ($s as $k => $v) {
+      $s[$k] = '!'.preg_quote($v).'!u';
+    }
+  }
+  return preg_replace($s,$r,$str);
+}
+
+/**
+ * Unicode aware replacement for ltrim()
+ *
+ * @author Andreas Gohr <andi@splitbrain.org>
+ * @see    ltrim()
+ * @return string
+ */
+function utf8_ltrim($str,$charlist=''){
+  if($charlist == '') return ltrim($str);
+  
+  //quote charlist for use in a characterclass
+  $charlist = preg_replace('!([\\\\\\-\\]\\[/])!','\\\${1}',$charlist);
+
+  return preg_replace('/^['.$charlist.']+/u','',$str);
+}
+
+/**
+ * Unicode aware replacement for rtrim()
+ *
+ * @author Andreas Gohr <andi@splitbrain.org>
+ * @see    rtrim()
+ * @return string
+ */
+function  utf8_rtrim($str,$charlist=''){
+  if($charlist == '') return rtrim($str);
+  
+  //quote charlist for use in a characterclass
+  $charlist = preg_replace('!([\\\\\\-\\]\\[/])!','\\\${1}',$charlist);
+  
+  return preg_replace('/['.$charlist.']+$/u','',$str);
+}
+
+/**
+ * Unicode aware replacement for trim()
+ *
+ * @author Andreas Gohr <andi@splitbrain.org>
+ * @see    trim()
+ * @return string
+ */
+function  utf8_trim($str,$charlist='') {
+  if($charlist == '') return trim($str);
+
+  return utf8_ltrim(utf8_rtrim($str));
+}
+
+
 /**
  * This is a unicode aware replacement for strtolower()
  *
@@ -123,7 +234,8 @@ function utf8_strtolower($string){
 
   global $UTF8_UPPER_TO_LOWER;
   $uni = utf8_to_unicode($string); 
-  for ($i=0; $i < count($uni); $i++){
+  $cnt = count($uni);
+  for ($i=0; $i < $cnt; $i++){
     if($UTF8_UPPER_TO_LOWER[$uni[$i]]){
       $uni[$i] = $UTF8_UPPER_TO_LOWER[$uni[$i]];
     }
@@ -142,11 +254,12 @@ function utf8_strtolower($string){
  */
 function utf8_strtoupper($string){
   if(!defined('UTF8_NOMBSTRING') && function_exists('mb_strtolower'))
-    return mb_strtolower($string,'utf-8');
+    return mb_strtoupper($string,'utf-8');
 
   global $UTF8_LOWER_TO_UPPER;
   $uni = utf8_to_unicode($string);
-  for ($i=0; $i < count($uni); $i++){
+  $cnt = count($uni);
+  for ($i=0; $i < $cnt; $i++){
     if($UTF8_LOWER_TO_UPPER[$uni[$i]]){
       $uni[$i] = $UTF8_LOWER_TO_UPPER[$uni[$i]];
     }
@@ -177,29 +290,23 @@ function utf8_deaccent($string,$case=0){
 /**
  * Removes special characters (nonalphanumeric) from a UTF-8 string
  *
- * Be sure to specify all specialchars you give in $repl in $keep, too
- * or it won't work.
- *
  * This function adds the controlchars 0x00 to 0x19 to the array of
  * stripped chars (they are not included in $UTF8_SPECIAL_CHARS)
  *
  * @author Andreas Gohr <andi@splitbrain.org>
- * @param  string $string The UTF8 string to strip of special chars
- * @param  string $repl   Replace special with this string
- * @param  string $keep   Special chars to keep (in UTF8)
+ * @param  string $string     The UTF8 string to strip of special chars
+ * @param  string $repl       Replace special with this string
+ * @param  string $additional Additional chars to strip (used in regexp char class)
  */
-function utf8_stripspecials($string,$repl='',$keep=''){
+function utf8_stripspecials($string,$repl='',$additional=''){
   global $UTF8_SPECIAL_CHARS;
-  if($keep != ''){
-    $specials = array_diff($UTF8_SPECIAL_CHARS, utf8_to_unicode($keep));
-  }else{
-    $specials = $UTF8_SPECIAL_CHARS;
+
+  static $specials = null;
+  if(is_null($specials)){
+    $specials = preg_quote(unicode_to_utf8($UTF8_SPECIAL_CHARS), '/');
   }
 
-  $specials = unicode_to_utf8($specials);
-  $specials = preg_quote($specials, '/');
-  
-  return preg_replace('/[\x00-\x19'.$specials.']/u',$repl,$string);
+  return preg_replace('/['.$additional.'\x00-\x19'.$specials.']/u',$repl,$string);
 }
 
 /**
@@ -207,38 +314,64 @@ function utf8_stripspecials($string,$repl='',$keep=''){
  *
  * Uses mb_string extension if available
  *
- * @author Scott Michael Reynen <scott@randomchaos.com>
- * @author Andreas Gohr <andi@splitbrain.org>
- * @link   http://www.randomchaos.com/document.php?source=php_and_unicode
+ * @author Harry Fuecks <hfuecks@gmail.com>
  * @see    strpos()
  */
 function utf8_strpos($haystack, $needle,$offset=0) {
   if(!defined('UTF8_NOMBSTRING') && function_exists('mb_strpos'))
     return mb_strpos($haystack,$needle,$offset,'utf-8');
 
-  $haystack = utf8_to_unicode($haystack);
-  $needle   = utf8_to_unicode($needle);
-  $position = $offset;
-  $found = false;
-  
-  while( (! $found ) && ( $position < count( $haystack ) ) ) {
-    if ( $needle[0] == $haystack[$position] ) {
-      for ($i = 1; $i < count( $needle ); $i++ ) {
-        if ( $needle[$i] != $haystack[ $position + $i ] ) break;
-      }
-      if ( $i == count( $needle ) ) {
-        $found = true;
-        $position--;
-      }
+  if(!$offset){
+    $ar = utf8_explode($needle, $str);
+    if ( count($ar) > 1 ) {
+       return utf8_strlen($ar[0]);
     }
-    $position++;
+    return false;
+  }else{
+    if ( !is_int($offset) ) {
+      trigger_error('Offset must be an integer',E_USER_WARNING);
+      return false;
+    }
+       
+    $str = utf8_substr($str, $offset);
+
+    if ( false !== ($pos = utf8_strpos($str,$needle))){
+       return $pos + $offset;
+    }
+    return false;
   }
-  return ( $found == true ) ? $position : false;
 }
 
 /**
- * This function will any UTF-8 encoded text and return it as
- * a list of Unicode values:
+ * Encodes UTF-8 characters to HTML entities
+ *
+ * @author <vpribish at shopping dot com>
+ * @link   http://www.php.net/manual/en/function.utf8-decode.php
+ */
+function utf8_tohtml ($str) {
+  $ret = '';
+  $max = strlen($str);
+  $last = 0;  // keeps the index of the last regular character
+  for ($i=0; $i<$max; $i++) {
+    $c = $str{$i};
+    $c1 = ord($c);
+    if ($c1>>5 == 6) {  // 110x xxxx, 110 prefix for 2 bytes unicode
+      $ret .= substr($str, $last, $i-$last); // append all the regular characters we've passed
+      $c1 &= 31; // remove the 3 bit two bytes prefix
+      $c2 = ord($str{++$i}); // the next byte
+      $c2 &= 63;  // remove the 2 bit trailing byte prefix
+      $c2 |= (($c1 & 3) << 6); // last 2 bits of c1 become first 2 of c2
+      $c1 >>= 2; // c1 shifts 2 to the right
+      $ret .= '&#' . ($c1 * 100 + $c2) . ';'; // this is the fastest string concatenation
+      $last = $i+1;
+    }
+  }
+  return $ret . substr($str, $last, $i); // append the last batch of regular characters
+} 
+
+/**
+ * This function returns any UTF-8 encoded text as a list of
+ * Unicode values:
  *
  * @author Scott Michael Reynen <scott@randomchaos.com>
  * @link   http://www.randomchaos.com/document.php?source=php_and_unicode
@@ -269,7 +402,7 @@ function utf8_to_unicode( $str ) {
 }
 
 /**
- * This function will convert a Unicode array back to its UTF-8 representation
+ * This function converts a Unicode array back to its UTF-8 representation
  *
  * @author Scott Michael Reynen <scott@randomchaos.com>
  * @link   http://www.randomchaos.com/document.php?source=php_and_unicode
@@ -419,15 +552,16 @@ $UTF8_UPPER_ACCENTS = array(
  * chars.
  *
  * The controlchars 0x00 to 0x19 are _not_ included in this array. The space 0x20 is!
+ * These chars are _not_ in the array either:  _ (0x5f), : 0x3a, . 0x2e, - 0x2d
  *
  * @author Andreas Gohr <andi@splitbrain.org>
  * @see    utf8_stripspecials()
  */
 $UTF8_SPECIAL_CHARS = array(
   0x001a, 0x001b, 0x001c, 0x001d, 0x001e, 0x001f, 0x0020, 0x0021, 0x0022, 0x0023,
-  0x0024, 0x0025, 0x0026, 0x0027, 0x0028, 0x0029, 0x002a, 0x002b, 0x002c, 0x002d,
-  0x002e, 0x002f, 0x003a, 0x003b, 0x003c, 0x003d, 0x003e, 0x003f, 0x0040, 0x005b,
-  0x005c, 0x005d, 0x005e, 0x005f, 0x0060, 0x0142, 0x007b, 0x007c, 0x007d, 0x007e,
+  0x0024, 0x0025, 0x0026, 0x0027, 0x0028, 0x0029, 0x002a, 0x002b, 0x002c,
+          0x002f,         0x003b, 0x003c, 0x003d, 0x003e, 0x003f, 0x0040, 0x005b,
+  0x005c, 0x005d, 0x005e,         0x0060, 0x007b, 0x007c, 0x007d, 0x007e,
   0x007f, 0x0080, 0x0081, 0x0082, 0x0083, 0x0084, 0x0085, 0x0086, 0x0087, 0x0088,
   0x0089, 0x008a, 0x008b, 0x008c, 0x008d, 0x008e, 0x008f, 0x0090, 0x0091, 0x0092,
 	0x0093, 0x0094, 0x0095, 0x0096, 0x0097, 0x0098, 0x0099, 0x009a, 0x009b, 0x009c,
@@ -475,4 +609,6 @@ $UTF8_SPECIAL_CHARS = array(
 	0xf8f1, 0xf8f2, 0xf8f3, 0xf8f4, 0xf8f5, 0xf8f6, 0xf8f7, 0xf8f8, 0xf8f9, 0xf8fa,
 	0xf8fb, 0xf8fc, 0xf8fd, 0xf8fe, 0xfe7c, 0xfe7d,
 );
-?>
+
+
+//Setup VIM: ex: et ts=2 enc=utf-8 :
